# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.currently_playing_context_object import CurrentlyPlayingContextObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_devices import ManyDevices
from spotifywebapiwithfixesandimprovementsfromsonallux.models.currently_playing_object import CurrentlyPlayingObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.cursor_paging_play_history_object import CursorPagingPlayHistoryObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.queue_object import QueueObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class PlayerController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(PlayerController, self).__init__(config)

    def get_information_about_the_users_current_playback(self,
                                                         market=None,
                                                         additional_types=None):
        """Does a GET request to /me/player.

        Get information about the user’s current playback state, including
        track or episode, progress, and active device.

        Args:
            market (str, optional): The request query parameter.
            additional_types (str, optional): The request query parameter.

        Returns:
            CurrentlyPlayingContextObject: Response from the API. Information
                about playback

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('additional_types')
                         .value(additional_types))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CurrentlyPlayingContextObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def transfer_a_users_playback(self,
                                  body=None):
        """Does a PUT request to /me/player.

        Transfer playback to a new device and optionally begin playback. This
        API only works for users who have Spotify Premium. The order of
        execution is not guaranteed when you use this API with other Player
        API endpoints.

        Args:
            body (MePlayerRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Playback transferred

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player')
            .http_method(HttpMethodEnum.PUT)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def get_a_users_available_devices(self):
        """Does a GET request to /me/player/devices.

        Get information about a user’s available Spotify Connect devices. Some
        device models are not supported and will not be listed in the API
        response.

        Returns:
            ManyDevices: Response from the API. A set of devices

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/devices')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyDevices.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_the_users_currently_playing_track(self,
                                              market=None,
                                              additional_types=None):
        """Does a GET request to /me/player/currently-playing.

        Get the object currently being played on the user's Spotify account.

        Args:
            market (str, optional): The request query parameter.
            additional_types (str, optional): The request query parameter.

        Returns:
            CurrentlyPlayingObject: Response from the API. Information about
                the currently playing track

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/currently-playing')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('additional_types')
                         .value(additional_types))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CurrentlyPlayingObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def start_a_users_playback(self,
                               device_id=None,
                               body=None):
        """Does a PUT request to /me/player/play.

        Start a new context or resume current playback on the user's active
        device. This API only works for users who have Spotify Premium. The
        order of execution is not guaranteed when you use this API with other
        Player API endpoints.

        Args:
            device_id (str, optional): The request query parameter.
            body (MePlayerPlayRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Playback started

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/play')
            .http_method(HttpMethodEnum.PUT)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def pause_a_users_playback(self,
                               device_id=None):
        """Does a PUT request to /me/player/pause.

        Pause playback on the user's account. This API only works for users
        who have Spotify Premium. The order of execution is not guaranteed
        when you use this API with other Player API endpoints.

        Args:
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Playback paused

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/pause')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def skip_users_playback_to_next_track(self,
                                          device_id=None):
        """Does a POST request to /me/player/next.

        Skips to next track in the user’s queue. This API only works for users
        who have Spotify Premium. The order of execution is not guaranteed
        when you use this API with other Player API endpoints.

        Args:
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command sent

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/next')
            .http_method(HttpMethodEnum.POST)
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def skip_users_playback_to_previous_track(self,
                                              device_id=None):
        """Does a POST request to /me/player/previous.

        Skips to previous track in the user’s queue. This API only works for
        users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command sent

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/previous')
            .http_method(HttpMethodEnum.POST)
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def seek_to_position_in_currently_playing_track(self,
                                                    position_ms,
                                                    device_id=None):
        """Does a PUT request to /me/player/seek.

        Seeks to the given position in the user’s currently playing track.
        This API only works for users who have Spotify Premium. The order of
        execution is not guaranteed when you use this API with other Player
        API endpoints.

        Args:
            position_ms (int): The request query parameter.
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command sent

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/seek')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('position_ms')
                         .value(position_ms))
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def set_repeat_mode_on_users_playback(self,
                                          state,
                                          device_id=None):
        """Does a PUT request to /me/player/repeat.

        Set the repeat mode for the user's playback. This API only works for
        users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
            state (str): The request query parameter.
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command sent

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/repeat')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('state')
                         .value(state))
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def set_volume_for_users_playback(self,
                                      volume_percent,
                                      device_id=None):
        """Does a PUT request to /me/player/volume.

        Set the volume for the user’s current playback device. This API only
        works for users who have Spotify Premium. The order of execution is
        not guaranteed when you use this API with other Player API endpoints.

        Args:
            volume_percent (int): The request query parameter.
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command sent

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/volume')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('volume_percent')
                         .value(volume_percent))
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def toggle_shuffle_for_users_playback(self,
                                          state,
                                          device_id=None):
        """Does a PUT request to /me/player/shuffle.

        Toggle shuffle on or off for user’s playback. This API only works for
        users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
            state (bool): The request query parameter.
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command sent

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/shuffle')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('state')
                         .value(state))
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()

    def get_recently_played(self,
                            limit=20,
                            after=None,
                            before=None):
        """Does a GET request to /me/player/recently-played.

        Get tracks from the current user's recently played tracks.
        _**Note**: Currently doesn't support podcast episodes._

        Args:
            limit (int, optional): The request query parameter. Example: 20
            after (int, optional): The request query parameter.
            before (int, optional): The request query parameter.

        Returns:
            CursorPagingPlayHistoryObject: Response from the API. A paged set
                of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/recently-played')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('after')
                         .value(after))
            .query_param(Parameter()
                         .key('before')
                         .value(before))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CursorPagingPlayHistoryObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_queue(self):
        """Does a GET request to /me/player/queue.

        Get the list of objects that make up the user's queue.

        Returns:
            QueueObject: Response from the API. Information about the queue

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/queue')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(QueueObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def add_to_queue(self,
                     uri,
                     device_id=None):
        """Does a POST request to /me/player/queue.

        Add an item to the end of the user's current playback queue. This API
        only works for users who have Spotify Premium. The order of execution
        is not guaranteed when you use this API with other Player API
        endpoints.

        Args:
            uri (str): The request query parameter.
            device_id (str, optional): The request query parameter.

        Returns:
            void: Response from the API. Command received

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/player/queue')
            .http_method(HttpMethodEnum.POST)
            .query_param(Parameter()
                         .key('uri')
                         .value(uri))
            .query_param(Parameter()
                         .key('device_id')
                         .value(device_id))
            .auth(Single('oauth_2_0'))
        ).execute()



__all__ = [
    'base_controller',
    'albums_controller',
    'artists_controller',
    'audiobooks_controller',
    'categories_controller',
    'chapters_controller',
    'episodes_controller',
    'genres_controller',
    'markets_controller',
    'player_controller',
    'playlists_controller',
    'search_controller',
    'shows_controller',
    'tracks_controller',
    'users_controller',
    'o_auth_authorization_controller',
]



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class Type3Enum(object):

    """Implementation of the 'Type3' enum.

    The object type.

    Attributes:
        USER: The enum member of type str.

    """
    _all_values = ['user']
    USER = 'user'

    @classmethod
    def validate(cls, value):
        """Validates value contains in enum

        Args:
            value: the value to be validated

        Returns:
            boolean : if value is valid enum values.

        """
        return value in cls._all_values
   


# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.artist_object import ArtistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_artists import ManyArtists
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_artist_discography_album_object import PagingArtistDiscographyAlbumObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_tracks import ManyTracks
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class ArtistsController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(ArtistsController, self).__init__(config)

    def get_an_artist(self,
                      id):
        """Does a GET request to /artists/{id}.

        Get Spotify catalog information for a single artist identified by
        their unique Spotify ID.

        Args:
            id (str): The request template parameter.

        Returns:
            ArtistObject: Response from the API. An artist

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/artists/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ArtistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_multiple_artists(self,
                             ids):
        """Does a GET request to /artists.

        Get Spotify catalog information for several artists based on their
        Spotify IDs.

        Args:
            ids (str): The request query parameter.

        Returns:
            ManyArtists: Response from the API. A set of artists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/artists')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyArtists.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_an_artists_albums(self,
                              id,
                              include_groups=None,
                              market=None,
                              limit=20,
                              offset=0):
        """Does a GET request to /artists/{id}/albums.

        Get Spotify catalog information about an artist's albums.

        Args:
            id (str): The request template parameter.
            include_groups (str, optional): The request query parameter.
            market (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingArtistDiscographyAlbumObject: Response from the API. Pages
                of albums

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/artists/{id}/albums')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('include_groups')
                         .value(include_groups))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingArtistDiscographyAlbumObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_an_artists_top_tracks(self,
                                  id,
                                  market=None):
        """Does a GET request to /artists/{id}/top-tracks.

        Get Spotify catalog information about an artist's top tracks by
        country.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.

        Returns:
            ManyTracks: Response from the API. A set of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/artists/{id}/top-tracks')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyTracks.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_an_artists_related_artists(self,
                                       id):
        """Does a GET request to /artists/{id}/related-artists.

        Get Spotify catalog information about artists similar to a given
        artist. Similarity is based on analysis of the Spotify community's
        listening history.

        Args:
            id (str): The request template parameter.

        Returns:
            ManyArtists: Response from the API. A set of artists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/artists/{id}/related-artists')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyArtists.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ItemType3Enum(object):

    """Implementation of the 'Item Type3' enum.

    The ID type: either `artist` or `user`.

    Attributes:
        ARTIST: The enum member of type str.
        USER: The enum member of type str.

    """
    ARTIST = 'artist'

    USER = 'user'




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class PlaylistTracksRefObject(object):

    """Implementation of the 'PlaylistTracksRefObject' model.

    Attributes:
        href (str): A link to the Web API endpoint where full details of the
            playlist's tracks can be retrieved.
        total (int): Number of tracks in the playlist.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "href": 'href',
        "total": 'total'
    }

    _optionals = [
        'href',
        'total',
    ]

    def __init__(self,
                 href=APIHelper.SKIP,
                 total=APIHelper.SKIP):
        """Constructor for the PlaylistTracksRefObject class"""

        # Initialize members of the class
        if href is not APIHelper.SKIP:
            self.href = href 
        if total is not APIHelper.SKIP:
            self.total = total 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        total = dictionary.get("total") if dictionary.get("total") else APIHelper.SKIP
        # Return an object of this model
        return cls(href,
                   total)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'total={(self.total if hasattr(self, "total") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'total={(self.total if hasattr(self, "total") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class UsersPlaylistsRequest(object):

    """Implementation of the 'Users Playlists Request' model.

    Attributes:
        name (str): The name for the new playlist, for example `"Your Coolest
            Playlist"`. This name does not need to be unique; a user may have
            several playlists with the same name.
        public (bool): Defaults to `true`. If `true` the playlist will be
            public, if `false` it will be private. To be able to create
            private playlists, the user must have granted the
            `playlist-modify-private`
            [scope](/documentation/web-api/concepts/scopes/#list-of-scopes)
        collaborative (bool): Defaults to `false`. If `true` the playlist will
            be collaborative. _**Note**: to create a collaborative playlist
            you must also set `public` to `false`. To create collaborative
            playlists you must have granted `playlist-modify-private` and
            `playlist-modify-public`
            [scopes](/documentation/web-api/concepts/scopes/#list-of-scopes)._
        description (str): value for playlist description as displayed in
            Spotify Clients and in the Web API.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "public": 'public',
        "collaborative": 'collaborative',
        "description": 'description'
    }

    _optionals = [
        'public',
        'collaborative',
        'description',
    ]

    def __init__(self,
                 name=None,
                 public=APIHelper.SKIP,
                 collaborative=APIHelper.SKIP,
                 description=APIHelper.SKIP):
        """Constructor for the UsersPlaylistsRequest class"""

        # Initialize members of the class
        self.name = name 
        if public is not APIHelper.SKIP:
            self.public = public 
        if collaborative is not APIHelper.SKIP:
            self.collaborative = collaborative 
        if description is not APIHelper.SKIP:
            self.description = description 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else None
        public = dictionary.get("public") if "public" in dictionary.keys() else APIHelper.SKIP
        collaborative = dictionary.get("collaborative") if "collaborative" in dictionary.keys() else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   public,
                   collaborative,
                   description)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={self.name!r}, '
                f'public={(self.public if hasattr(self, "public") else None)!r}, '
                f'collaborative={(self.collaborative if hasattr(self, "collaborative") else None)!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={self.name!s}, '
                f'public={(self.public if hasattr(self, "public") else None)!s}, '
                f'collaborative={(self.collaborative if hasattr(self, "collaborative") else None)!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class EpisodeRestrictionObject(object):

    """Implementation of the 'EpisodeRestrictionObject' model.

    Attributes:
        reason (str): The reason for the restriction. Supported values: -
            `market` - The content item is not available in the given market.
            - `product` - The content item is not available for the user's
            subscription type. - `explicit` - The content item is explicit and
            the user's account is set to not play explicit content. 
            Additional reasons may be added in the future. **Note**: If you
            use this field, make sure that your application safely handles
            unknown values.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "reason": 'reason'
    }

    _optionals = [
        'reason',
    ]

    def __init__(self,
                 reason=APIHelper.SKIP):
        """Constructor for the EpisodeRestrictionObject class"""

        # Initialize members of the class
        if reason is not APIHelper.SKIP:
            self.reason = reason 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        reason = dictionary.get("reason") if dictionary.get("reason") else APIHelper.SKIP
        # Return an object of this model
        return cls(reason)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return True

        if not isinstance(dictionary, dict):
            return False

        return True

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'reason={(self.reason if hasattr(self, "reason") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'reason={(self.reason if hasattr(self, "reason") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.playlist_object import PlaylistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_playlist_track_object import PagingPlaylistTrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.playlist_snapshot_id import PlaylistSnapshotId
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_playlist_object import PagingPlaylistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_featured_playlist_object import PagingFeaturedPlaylistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.image_object import ImageObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class PlaylistsController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(PlaylistsController, self).__init__(config)

    def get_playlist(self,
                     playlist_id,
                     market=None,
                     fields=None,
                     additional_types=None):
        """Does a GET request to /playlists/{playlist_id}.

        Get a playlist owned by a Spotify user.

        Args:
            playlist_id (str): The request template parameter.
            market (str, optional): The request query parameter.
            fields (str, optional): The request query parameter.
            additional_types (str, optional): The request query parameter.

        Returns:
            PlaylistObject: Response from the API. A playlist

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('fields')
                         .value(fields))
            .query_param(Parameter()
                         .key('additional_types')
                         .value(additional_types))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PlaylistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def change_playlist_details(self,
                                playlist_id,
                                body=None):
        """Does a PUT request to /playlists/{playlist_id}.

        Change a playlist's name and public/private state. (The user must, of
        course, own the playlist.)

        Args:
            playlist_id (str): The request template parameter.
            body (PlaylistsRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Playlist updated

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def get_playlists_tracks(self,
                             playlist_id,
                             market=None,
                             fields=None,
                             limit=20,
                             offset=0,
                             additional_types=None):
        """Does a GET request to /playlists/{playlist_id}/tracks.

        Get full details of the items of a playlist owned by a Spotify user.

        Args:
            playlist_id (str): The request template parameter.
            market (str, optional): The request query parameter.
            fields (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0
            additional_types (str, optional): The request query parameter.

        Returns:
            PagingPlaylistTrackObject: Response from the API. Pages of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/tracks')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('fields')
                         .value(fields))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .query_param(Parameter()
                         .key('additional_types')
                         .value(additional_types))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingPlaylistTrackObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def add_tracks_to_playlist(self,
                               playlist_id,
                               position=None,
                               uris=None,
                               body=None):
        """Does a POST request to /playlists/{playlist_id}/tracks.

        Add one or more items to a user's playlist.

        Args:
            playlist_id (str): The request template parameter.
            position (int, optional): The request query parameter.
            uris (str, optional): The request query parameter.
            body (PlaylistsTracksRequest, optional): The request body
                parameter.

        Returns:
            PlaylistSnapshotId: Response from the API. A snapshot ID for the
                playlist

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/tracks')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .query_param(Parameter()
                         .key('position')
                         .value(position))
            .query_param(Parameter()
                         .key('uris')
                         .value(uris))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PlaylistSnapshotId.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def reorder_or_replace_playlists_tracks(self,
                                            playlist_id,
                                            uris=None,
                                            body=None):
        """Does a PUT request to /playlists/{playlist_id}/tracks.

        Either reorder or replace items in a playlist depending on the
        request's parameters.
        To reorder items, include `range_start`, `insert_before`,
        `range_length` and `snapshot_id` in the request's body.
        To replace items, include `uris` as either a query parameter or in the
        request's body.
        Replacing items in a playlist will overwrite its existing items. This
        operation can be used for replacing or clearing items in a playlist.
        <br/>
        **Note**: Replace and reorder are mutually exclusive operations which
        share the same endpoint, but have different parameters.
        These operations can't be applied together in a single request.

        Args:
            playlist_id (str): The request template parameter.
            uris (str, optional): The request query parameter.
            body (PlaylistsTracksRequest1, optional): The request body
                parameter.

        Returns:
            PlaylistSnapshotId: Response from the API. A snapshot ID for the
                playlist

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/tracks')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .query_param(Parameter()
                         .key('uris')
                         .value(uris))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PlaylistSnapshotId.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def remove_tracks_playlist(self,
                               playlist_id,
                               body=None):
        """Does a DELETE request to /playlists/{playlist_id}/tracks.

        Remove one or more items from a user's playlist.

        Args:
            playlist_id (str): The request template parameter.
            body (PlaylistsTracksRequest2, optional): The request body
                parameter.

        Returns:
            PlaylistSnapshotId: Response from the API. A snapshot ID for the
                playlist

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/tracks')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PlaylistSnapshotId.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_a_list_of_current_users_playlists(self,
                                              limit=20,
                                              offset=0):
        """Does a GET request to /me/playlists.

        Get a list of the playlists owned or followed by the current Spotify
        user.

        Args:
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingPlaylistObject: Response from the API. A paged set of
                playlists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/playlists')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingPlaylistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_list_users_playlists(self,
                                 user_id,
                                 limit=20,
                                 offset=0):
        """Does a GET request to /users/{user_id}/playlists.

        Get a list of the playlists owned or followed by a Spotify user.

        Args:
            user_id (str): The request template parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingPlaylistObject: Response from the API. A paged set of
                playlists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/users/{user_id}/playlists')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('user_id')
                            .value(user_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingPlaylistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def create_playlist(self,
                        user_id,
                        body=None):
        """Does a POST request to /users/{user_id}/playlists.

        Create a playlist for a Spotify user. (The playlist will be empty until
        you [add
        tracks](/documentation/web-api/reference/add-tracks-to-playlist).)
        Each user is generally limited to a maximum of 11000 playlists.

        Args:
            user_id (str): The request template parameter.
            body (UsersPlaylistsRequest, optional): The request body parameter.

        Returns:
            PlaylistObject: Response from the API. A playlist

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/users/{user_id}/playlists')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('user_id')
                            .value(user_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PlaylistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_featured_playlists(self,
                               locale=None,
                               limit=20,
                               offset=0):
        """Does a GET request to /browse/featured-playlists.

        Get a list of Spotify featured playlists (shown, for example, on a
        Spotify player's 'Browse' tab).

        Args:
            locale (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingFeaturedPlaylistObject: Response from the API. A paged set
                of playlists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/browse/featured-playlists')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('locale')
                         .value(locale))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingFeaturedPlaylistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_a_categories_playlists(self,
                                   category_id,
                                   limit=20,
                                   offset=0):
        """Does a GET request to /browse/categories/{category_id}/playlists.

        Get a list of Spotify playlists tagged with a particular category.

        Args:
            category_id (str): The request template parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingFeaturedPlaylistObject: Response from the API. A paged set
                of playlists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/browse/categories/{category_id}/playlists')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('category_id')
                            .value(category_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingFeaturedPlaylistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_playlist_cover(self,
                           playlist_id):
        """Does a GET request to /playlists/{playlist_id}/images.

        Get the current image associated with a specific playlist.

        Args:
            playlist_id (str): The request template parameter.

        Returns:
            List[ImageObject]: Response from the API. A set of images

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/images')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ImageObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def upload_custom_playlist_cover(self,
                                     playlist_id,
                                     body):
        """Does a PUT request to /playlists/{playlist_id}/images.

        Replace the image used to represent a specific playlist.

        Args:
            playlist_id (str): The request template parameter.
            body (Any): The request body parameter.

        Returns:
            void: Response from the API. Image uploaded

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/images')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('content-type')
                          .value('application/json; charset=utf-8'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class CopyrightObject(object):

    """Implementation of the 'CopyrightObject' model.

    Attributes:
        text (str): The copyright text for this content.
        mtype (str): The type of copyright: `C` = the copyright, `P` = the
            sound recording (performance) copyright.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "text": 'text',
        "mtype": 'type'
    }

    _optionals = [
        'text',
        'mtype',
    ]

    def __init__(self,
                 text=APIHelper.SKIP,
                 mtype=APIHelper.SKIP):
        """Constructor for the CopyrightObject class"""

        # Initialize members of the class
        if text is not APIHelper.SKIP:
            self.text = text 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        text = dictionary.get("text") if dictionary.get("text") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        # Return an object of this model
        return cls(text,
                   mtype)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return True

        if not isinstance(dictionary, dict):
            return False

        return True

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'text={(self.text if hasattr(self, "text") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'text={(self.text if hasattr(self, "text") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.image_object import ImageObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.playlist_owner_object import PlaylistOwnerObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.playlist_tracks_ref_object import PlaylistTracksRefObject


class SimplifiedPlaylistObject(object):

    """Implementation of the 'SimplifiedPlaylistObject' model.

    Attributes:
        collaborative (bool): `true` if the owner allows other users to modify
            the playlist.
        description (str): The playlist description. _Only returned for
            modified, verified playlists, otherwise_ `null`.
        external_urls (ExternalUrlObject): Known external URLs for this
            playlist.
        href (str): A link to the Web API endpoint providing full details of
            the playlist.
        id (str): The [Spotify
            ID](/documentation/web-api/concepts/spotify-uris-ids) for the
            playlist.
        images (List[ImageObject]): Images for the playlist. The array may be
            empty or contain up to three images. The images are returned by
            size in descending order. See [Working with
            Playlists](/documentation/web-api/concepts/playlists). _**Note**:
            If returned, the source URL for the image (`url`) is temporary and
            will expire in less than a day._
        name (str): The name of the playlist.
        owner (PlaylistOwnerObject): The user who owns the playlist
        public (bool): The playlist's public/private status: `true` the
            playlist is public, `false` the playlist is private, `null` the
            playlist status is not relevant. For more about public/private
            status, see [Working with
            Playlists](/documentation/web-api/concepts/playlists)
        snapshot_id (str): The version identifier for the current playlist.
            Can be supplied in other requests to target a specific playlist
            version
        tracks (PlaylistTracksRefObject): A collection containing a link (
            `href` ) to the Web API endpoint where full details of the
            playlist's tracks can be retrieved, along with the `total` number
            of tracks in the playlist. Note, a track object may be `null`.
            This can happen if a track is no longer available.
        mtype (str): The object type: "playlist"
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for the
            playlist.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "collaborative": 'collaborative',
        "description": 'description',
        "external_urls": 'external_urls',
        "href": 'href',
        "id": 'id',
        "images": 'images',
        "name": 'name',
        "owner": 'owner',
        "public": 'public',
        "snapshot_id": 'snapshot_id',
        "tracks": 'tracks',
        "mtype": 'type',
        "uri": 'uri'
    }

    _optionals = [
        'collaborative',
        'description',
        'external_urls',
        'href',
        'id',
        'images',
        'name',
        'owner',
        'public',
        'snapshot_id',
        'tracks',
        'mtype',
        'uri',
    ]

    def __init__(self,
                 collaborative=APIHelper.SKIP,
                 description=APIHelper.SKIP,
                 external_urls=APIHelper.SKIP,
                 href=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 images=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 owner=APIHelper.SKIP,
                 public=APIHelper.SKIP,
                 snapshot_id=APIHelper.SKIP,
                 tracks=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 uri=APIHelper.SKIP):
        """Constructor for the SimplifiedPlaylistObject class"""

        # Initialize members of the class
        if collaborative is not APIHelper.SKIP:
            self.collaborative = collaborative 
        if description is not APIHelper.SKIP:
            self.description = description 
        if external_urls is not APIHelper.SKIP:
            self.external_urls = external_urls 
        if href is not APIHelper.SKIP:
            self.href = href 
        if id is not APIHelper.SKIP:
            self.id = id 
        if images is not APIHelper.SKIP:
            self.images = images 
        if name is not APIHelper.SKIP:
            self.name = name 
        if owner is not APIHelper.SKIP:
            self.owner = owner 
        if public is not APIHelper.SKIP:
            self.public = public 
        if snapshot_id is not APIHelper.SKIP:
            self.snapshot_id = snapshot_id 
        if tracks is not APIHelper.SKIP:
            self.tracks = tracks 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if uri is not APIHelper.SKIP:
            self.uri = uri 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        collaborative = dictionary.get("collaborative") if "collaborative" in dictionary.keys() else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if 'external_urls' in dictionary.keys() else APIHelper.SKIP
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        images = None
        if dictionary.get('images') is not None:
            images = [ImageObject.from_dictionary(x) for x in dictionary.get('images')]
        else:
            images = APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        owner = PlaylistOwnerObject.from_dictionary(dictionary.get('owner')) if 'owner' in dictionary.keys() else APIHelper.SKIP
        public = dictionary.get("public") if "public" in dictionary.keys() else APIHelper.SKIP
        snapshot_id = dictionary.get("snapshot_id") if dictionary.get("snapshot_id") else APIHelper.SKIP
        tracks = PlaylistTracksRefObject.from_dictionary(dictionary.get('tracks')) if 'tracks' in dictionary.keys() else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        uri = dictionary.get("uri") if dictionary.get("uri") else APIHelper.SKIP
        # Return an object of this model
        return cls(collaborative,
                   description,
                   external_urls,
                   href,
                   id,
                   images,
                   name,
                   owner,
                   public,
                   snapshot_id,
                   tracks,
                   mtype,
                   uri)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'collaborative={(self.collaborative if hasattr(self, "collaborative") else None)!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r}, '
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!r}, '
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'images={(self.images if hasattr(self, "images") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'owner={(self.owner if hasattr(self, "owner") else None)!r}, '
                f'public={(self.public if hasattr(self, "public") else None)!r}, '
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!r}, '
                f'tracks={(self.tracks if hasattr(self, "tracks") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'collaborative={(self.collaborative if hasattr(self, "collaborative") else None)!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s}, '
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!s}, '
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'images={(self.images if hasattr(self, "images") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'owner={(self.owner if hasattr(self, "owner") else None)!s}, '
                f'public={(self.public if hasattr(self, "public") else None)!s}, '
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!s}, '
                f'tracks={(self.tracks if hasattr(self, "tracks") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class ResumePointObject(object):

    """Implementation of the 'ResumePointObject' model.

    Attributes:
        fully_played (bool): Whether or not the episode has been fully played
            by the user.
        resume_position_ms (int): The user's most recent position in the
            episode in milliseconds.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "fully_played": 'fully_played',
        "resume_position_ms": 'resume_position_ms'
    }

    _optionals = [
        'fully_played',
        'resume_position_ms',
    ]

    def __init__(self,
                 fully_played=APIHelper.SKIP,
                 resume_position_ms=APIHelper.SKIP):
        """Constructor for the ResumePointObject class"""

        # Initialize members of the class
        if fully_played is not APIHelper.SKIP:
            self.fully_played = fully_played 
        if resume_position_ms is not APIHelper.SKIP:
            self.resume_position_ms = resume_position_ms 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        fully_played = dictionary.get("fully_played") if "fully_played" in dictionary.keys() else APIHelper.SKIP
        resume_position_ms = dictionary.get("resume_position_ms") if dictionary.get("resume_position_ms") else APIHelper.SKIP
        # Return an object of this model
        return cls(fully_played,
                   resume_position_ms)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return True

        if not isinstance(dictionary, dict):
            return False

        return True

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'fully_played={(self.fully_played if hasattr(self, "fully_played") else None)!r}, '
                f'resume_position_ms={(self.resume_position_ms if hasattr(self, "resume_position_ms") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'fully_played={(self.fully_played if hasattr(self, "fully_played") else None)!s}, '
                f'resume_position_ms={(self.resume_position_ms if hasattr(self, "resume_position_ms") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.audiobook_object import AudiobookObject


class ManyAudiobooks(object):

    """Implementation of the 'ManyAudiobooks' model.

    Attributes:
        audiobooks (List[AudiobookObject]): The model property of type
            List[AudiobookObject].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "audiobooks": 'audiobooks'
    }

    def __init__(self,
                 audiobooks=None):
        """Constructor for the ManyAudiobooks class"""

        # Initialize members of the class
        self.audiobooks = audiobooks 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        audiobooks = None
        if dictionary.get('audiobooks') is not None:
            audiobooks = [AudiobookObject.from_dictionary(x) for x in dictionary.get('audiobooks')]
        # Return an object of this model
        return cls(audiobooks)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'audiobooks={self.audiobooks!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'audiobooks={self.audiobooks!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class QueueObject(object):

    """Implementation of the 'QueueObject' model.

    Attributes:
        currently_playing (TrackObject | EpisodeObject | None): The currently
            playing track or episode. Can be `null`.
        queue (List[TrackObject | EpisodeObject] | None): The tracks or
            episodes in the queue. Can be empty.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "currently_playing": 'currently_playing',
        "queue": 'queue'
    }

    _optionals = [
        'currently_playing',
        'queue',
    ]

    def __init__(self,
                 currently_playing=APIHelper.SKIP,
                 queue=APIHelper.SKIP):
        """Constructor for the QueueObject class"""

        # Initialize members of the class
        if currently_playing is not APIHelper.SKIP:
            self.currently_playing = currently_playing 
        if queue is not APIHelper.SKIP:
            self.queue = queue 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        from spotifywebapiwithfixesandimprovementsfromsonallux.utilities.union_type_lookup import UnionTypeLookUp

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        currently_playing = APIHelper.deserialize_union_type(UnionTypeLookUp.get('QueueObjectCurrentlyPlaying'), dictionary.get('currently_playing'), False) if dictionary.get('currently_playing') is not None else APIHelper.SKIP
        queue = APIHelper.deserialize_union_type(UnionTypeLookUp.get('QueueObjectQueue'), dictionary.get('queue'), False) if dictionary.get('queue') is not None else APIHelper.SKIP
        # Return an object of this model
        return cls(currently_playing,
                   queue)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'currently_playing={(self.currently_playing if hasattr(self, "currently_playing") else None)!r}, '
                f'queue={(self.queue if hasattr(self, "queue") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'currently_playing={(self.currently_playing if hasattr(self, "currently_playing") else None)!s}, '
                f'queue={(self.queue if hasattr(self, "queue") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class Type5Enum(object):

    """Implementation of the 'Type5' enum.

    The object type.

    Attributes:
        AUDIO_FEATURES: The enum member of type str.

    """
    AUDIO_FEATURES = 'audio_features'




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class OAuthToken(object):

    """Implementation of the 'OAuthToken' model.

    OAuth 2 Authorization endpoint response

    Attributes:
        access_token (str): Access token
        token_type (str): Type of access token
        expires_in (int): Time in seconds before the access token expires
        scope (str): List of scopes granted This is a space-delimited list of
            strings.
        expiry (int): Time of token expiry as unix timestamp (UTC)
        refresh_token (str): Refresh token Used to get a new access token when
            it expires.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "access_token": 'access_token',
        "token_type": 'token_type',
        "expires_in": 'expires_in',
        "scope": 'scope',
        "expiry": 'expiry',
        "refresh_token": 'refresh_token'
    }

    _optionals = [
        'expires_in',
        'scope',
        'expiry',
        'refresh_token',
    ]

    def __init__(self,
                 access_token=None,
                 token_type=None,
                 expires_in=APIHelper.SKIP,
                 scope=APIHelper.SKIP,
                 expiry=APIHelper.SKIP,
                 refresh_token=APIHelper.SKIP):
        """Constructor for the OAuthToken class"""

        # Initialize members of the class
        self.access_token = access_token 
        self.token_type = token_type 
        if expires_in is not APIHelper.SKIP:
            self.expires_in = expires_in 
        if scope is not APIHelper.SKIP:
            self.scope = scope 
        if expiry is not APIHelper.SKIP:
            self.expiry = expiry 
        if refresh_token is not APIHelper.SKIP:
            self.refresh_token = refresh_token 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        access_token = dictionary.get("access_token") if dictionary.get("access_token") else None
        token_type = dictionary.get("token_type") if dictionary.get("token_type") else None
        expires_in = dictionary.get("expires_in") if dictionary.get("expires_in") else APIHelper.SKIP
        scope = dictionary.get("scope") if dictionary.get("scope") else APIHelper.SKIP
        expiry = dictionary.get("expiry") if dictionary.get("expiry") else APIHelper.SKIP
        refresh_token = dictionary.get("refresh_token") if dictionary.get("refresh_token") else APIHelper.SKIP
        # Return an object of this model
        return cls(access_token,
                   token_type,
                   expires_in,
                   scope,
                   expiry,
                   refresh_token)



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.show_object import ShowObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_simplified_shows import ManySimplifiedShows
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_simplified_episode_object import PagingSimplifiedEpisodeObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_saved_show_object import PagingSavedShowObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class ShowsController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(ShowsController, self).__init__(config)

    def get_a_show(self,
                   id,
                   market=None):
        """Does a GET request to /shows/{id}.

        Get Spotify catalog information for a single show identified by its
        unique Spotify ID.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.

        Returns:
            ShowObject: Response from the API. A show

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/shows/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ShowObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_multiple_shows(self,
                           ids,
                           market=None):
        """Does a GET request to /shows.

        Get Spotify catalog information for several shows based on their
        Spotify IDs.

        Args:
            ids (str): The request query parameter.
            market (str, optional): The request query parameter.

        Returns:
            ManySimplifiedShows: Response from the API. A set of shows

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/shows')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManySimplifiedShows.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_a_shows_episodes(self,
                             id,
                             market=None,
                             limit=20,
                             offset=0):
        """Does a GET request to /shows/{id}/episodes.

        Get Spotify catalog information about an show’s episodes. Optional
        parameters can be used to limit the number of episodes returned.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingSimplifiedEpisodeObject: Response from the API. Pages of
                episodes

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/shows/{id}/episodes')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingSimplifiedEpisodeObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_saved_shows(self,
                              limit=20,
                              offset=0):
        """Does a GET request to /me/shows.

        Get a list of shows saved in the current Spotify user's library.
        Optional parameters can be used to limit the number of shows returned.

        Args:
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingSavedShowObject: Response from the API. Pages of shows

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/shows')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingSavedShowObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def save_shows_user(self,
                        ids,
                        body=None):
        """Does a PUT request to /me/shows.

        Save one or more shows to current Spotify user's library.

        Args:
            ids (str): The request query parameter.
            body (MeShowsRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Show saved

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/shows')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def remove_shows_user(self,
                          ids,
                          market=None,
                          body=None):
        """Does a DELETE request to /me/shows.

        Delete one or more shows from current Spotify user's library.

        Args:
            ids (str): The request query parameter.
            market (str, optional): The request query parameter.
            body (MeShowsRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Show removed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/shows')
            .http_method(HttpMethodEnum.DELETE)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def check_users_saved_shows(self,
                                ids):
        """Does a GET request to /me/shows/contains.

        Check if one or more shows is already saved in the current Spotify
        user's library.

        Args:
            ids (str): The request query parameter.

        Returns:
            List[bool]: Response from the API. Array of booleans

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/shows/contains')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class PlaylistsTracksRequest1(object):

    """Implementation of the 'Playlists Tracks Request1' model.

    Attributes:
        uris (List[str]): The model property of type List[str].
        range_start (int): The position of the first item to be reordered.
        insert_before (int): The position where the items should be
            inserted.<br/>To reorder the items to the end of the playlist,
            simply set _insert_before_ to the position after the last
            item.<br/>Examples:<br/>To reorder the first item to the last
            position in a playlist with 10 items, set _range_start_ to 0, and
            _insert_before_ to 10.<br/>To reorder the last item in a playlist
            with 10 items to the start of the playlist, set _range_start_ to
            9, and _insert_before_ to 0.
        range_length (int): The amount of items to be reordered. Defaults to 1
            if not set.<br/>The range of items to be reordered begins from the
            _range_start_ position, and includes the _range_length_ subsequent
            items.<br/>Example:<br/>To move the items at index 9-10 to the
            start of the playlist, _range_start_ is set to 9, and
            _range_length_ is set to 2.
        snapshot_id (str): The playlist's snapshot ID against which you want
            to make the changes.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "uris": 'uris',
        "range_start": 'range_start',
        "insert_before": 'insert_before',
        "range_length": 'range_length',
        "snapshot_id": 'snapshot_id'
    }

    _optionals = [
        'uris',
        'range_start',
        'insert_before',
        'range_length',
        'snapshot_id',
    ]

    def __init__(self,
                 uris=APIHelper.SKIP,
                 range_start=APIHelper.SKIP,
                 insert_before=APIHelper.SKIP,
                 range_length=APIHelper.SKIP,
                 snapshot_id=APIHelper.SKIP):
        """Constructor for the PlaylistsTracksRequest1 class"""

        # Initialize members of the class
        if uris is not APIHelper.SKIP:
            self.uris = uris 
        if range_start is not APIHelper.SKIP:
            self.range_start = range_start 
        if insert_before is not APIHelper.SKIP:
            self.insert_before = insert_before 
        if range_length is not APIHelper.SKIP:
            self.range_length = range_length 
        if snapshot_id is not APIHelper.SKIP:
            self.snapshot_id = snapshot_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        uris = dictionary.get("uris") if dictionary.get("uris") else APIHelper.SKIP
        range_start = dictionary.get("range_start") if dictionary.get("range_start") else APIHelper.SKIP
        insert_before = dictionary.get("insert_before") if dictionary.get("insert_before") else APIHelper.SKIP
        range_length = dictionary.get("range_length") if dictionary.get("range_length") else APIHelper.SKIP
        snapshot_id = dictionary.get("snapshot_id") if dictionary.get("snapshot_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(uris,
                   range_start,
                   insert_before,
                   range_length,
                   snapshot_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'uris={(self.uris if hasattr(self, "uris") else None)!r}, '
                f'range_start={(self.range_start if hasattr(self, "range_start") else None)!r}, '
                f'insert_before={(self.insert_before if hasattr(self, "insert_before") else None)!r}, '
                f'range_length={(self.range_length if hasattr(self, "range_length") else None)!r}, '
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'uris={(self.uris if hasattr(self, "uris") else None)!s}, '
                f'range_start={(self.range_start if hasattr(self, "range_start") else None)!s}, '
                f'insert_before={(self.insert_before if hasattr(self, "insert_before") else None)!s}, '
                f'range_length={(self.range_length if hasattr(self, "range_length") else None)!s}, '
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_1 import Track1


class PlaylistsTracksRequest2(object):

    """Implementation of the 'Playlists Tracks Request2' model.

    Attributes:
        tracks (List[Track1]): An array of objects containing [Spotify
            URIs](/documentation/web-api/concepts/spotify-uris-ids) of the
            tracks or episodes to remove. For example: `{ "tracks": [{ "uri":
            "spotify:track:4iV5W9uYEdYUVa79Axb7Rh" },{ "uri":
            "spotify:track:1301WleyT98MSxVHPZCA6M" }] }`. A maximum of 100
            objects can be sent at once.
        snapshot_id (str): The playlist's snapshot ID against which you want
            to make the changes. The API will validate that the specified
            items exist and in the specified positions and make the changes,
            even if more recent changes have been made to the playlist.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tracks": 'tracks',
        "snapshot_id": 'snapshot_id'
    }

    _optionals = [
        'snapshot_id',
    ]

    def __init__(self,
                 tracks=None,
                 snapshot_id=APIHelper.SKIP):
        """Constructor for the PlaylistsTracksRequest2 class"""

        # Initialize members of the class
        self.tracks = tracks 
        if snapshot_id is not APIHelper.SKIP:
            self.snapshot_id = snapshot_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        tracks = None
        if dictionary.get('tracks') is not None:
            tracks = [Track1.from_dictionary(x) for x in dictionary.get('tracks')]
        snapshot_id = dictionary.get("snapshot_id") if dictionary.get("snapshot_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(tracks,
                   snapshot_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'tracks={self.tracks!r}, '
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'tracks={self.tracks!s}, '
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ReasonEnum(object):

    """Implementation of the 'Reason' enum.

    The reason for the restriction. Albums may be restricted if the content is
    not available in a given market, to the user's subscription type, or when
    the user's account is set to not play explicit content.
    Additional reasons may be added in the future.

    Attributes:
        MARKET: The enum member of type str.
        PRODUCT: The enum member of type str.
        EXPLICIT: The enum member of type str.

    """
    _all_values = ['market', 'product', 'explicit']
    MARKET = 'market'

    PRODUCT = 'product'

    EXPLICIT = 'explicit'

    @classmethod
    def validate(cls, value):
        """Validates value contains in enum

        Args:
            value: the value to be validated

        Returns:
            boolean : if value is valid enum values.

        """
        return value in cls._all_values
   


# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OAuthProviderErrorEnum(object):

    """Implementation of the 'OAuthProviderError' enum.

    OAuth 2 Authorization error codes

    Attributes:
        INVALID_REQUEST: The request is missing a required parameter, includes
            an unsupported parameter value (other than grant type), repeats a
            parameter, includes multiple credentials, utilizes more than one
            mechanism for authenticating the client, or is otherwise malformed.
        INVALID_CLIENT: Client authentication failed (e.g., unknown client, no
            client authentication included, or unsupported authentication
            method).
        INVALID_GRANT: The provided authorization grant (e.g., authorization
            code, resource owner credentials) or refresh token is invalid,
            expired, revoked, does not match the redirection URI used in the
            authorization request, or was issued to another client.
        UNAUTHORIZED_CLIENT: The authenticated client is not authorized to use
            this authorization grant type.
        UNSUPPORTED_GRANT_TYPE: The authorization grant type is not supported
            by the authorization server.
        INVALID_SCOPE: The requested scope is invalid, unknown, malformed, or
            exceeds the scope granted by the resource owner.

    """
    INVALID_REQUEST = 'invalid_request'

    INVALID_CLIENT = 'invalid_client'

    INVALID_GRANT = 'invalid_grant'

    UNAUTHORIZED_CLIENT = 'unauthorized_client'

    UNSUPPORTED_GRANT_TYPE = 'unsupported_grant_type'

    INVALID_SCOPE = 'invalid_scope'




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class AlbumTypeEnum(object):

    """Implementation of the 'AlbumType' enum.

    The type of the album.

    Attributes:
        ALBUM: The enum member of type str.
        SINGLE: The enum member of type str.
        COMPILATION: The enum member of type str.

    """
    _all_values = ['album', 'single', 'compilation']
    ALBUM = 'album'

    SINGLE = 'single'

    COMPILATION = 'compilation'

    @classmethod
    def validate(cls, value):
        """Validates value contains in enum

        Args:
            value: the value to be validated

        Returns:
            boolean : if value is valid enum values.

        """
        return value in cls._all_values
   


# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.meta import Meta
from spotifywebapiwithfixesandimprovementsfromsonallux.models.section_object import SectionObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.segment_object import SegmentObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.time_interval_object import TimeIntervalObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track import Track


class AudioAnalysisObject(object):

    """Implementation of the 'AudioAnalysisObject' model.

    Attributes:
        meta (Meta): The model property of type Meta.
        track (Track): The model property of type Track.
        bars (List[TimeIntervalObject]): The time intervals of the bars
            throughout the track. A bar (or measure) is a segment of time
            defined as a given number of beats.
        beats (List[TimeIntervalObject]): The time intervals of beats
            throughout the track. A beat is the basic time unit of a piece of
            music; for example, each tick of a metronome. Beats are typically
            multiples of tatums.
        sections (List[SectionObject]): Sections are defined by large
            variations in rhythm or timbre, e.g. chorus, verse, bridge, guitar
            solo, etc. Each section contains its own descriptions of tempo,
            key, mode, time_signature, and loudness.
        segments (List[SegmentObject]): Each segment contains a roughly
            conisistent sound throughout its duration.
        tatums (List[TimeIntervalObject]): A tatum represents the lowest
            regular pulse train that a listener intuitively infers from the
            timing of perceived musical events (segments).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "meta": 'meta',
        "track": 'track',
        "bars": 'bars',
        "beats": 'beats',
        "sections": 'sections',
        "segments": 'segments',
        "tatums": 'tatums'
    }

    _optionals = [
        'meta',
        'track',
        'bars',
        'beats',
        'sections',
        'segments',
        'tatums',
    ]

    def __init__(self,
                 meta=APIHelper.SKIP,
                 track=APIHelper.SKIP,
                 bars=APIHelper.SKIP,
                 beats=APIHelper.SKIP,
                 sections=APIHelper.SKIP,
                 segments=APIHelper.SKIP,
                 tatums=APIHelper.SKIP):
        """Constructor for the AudioAnalysisObject class"""

        # Initialize members of the class
        if meta is not APIHelper.SKIP:
            self.meta = meta 
        if track is not APIHelper.SKIP:
            self.track = track 
        if bars is not APIHelper.SKIP:
            self.bars = bars 
        if beats is not APIHelper.SKIP:
            self.beats = beats 
        if sections is not APIHelper.SKIP:
            self.sections = sections 
        if segments is not APIHelper.SKIP:
            self.segments = segments 
        if tatums is not APIHelper.SKIP:
            self.tatums = tatums 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        meta = Meta.from_dictionary(dictionary.get('meta')) if 'meta' in dictionary.keys() else APIHelper.SKIP
        track = Track.from_dictionary(dictionary.get('track')) if 'track' in dictionary.keys() else APIHelper.SKIP
        bars = None
        if dictionary.get('bars') is not None:
            bars = [TimeIntervalObject.from_dictionary(x) for x in dictionary.get('bars')]
        else:
            bars = APIHelper.SKIP
        beats = None
        if dictionary.get('beats') is not None:
            beats = [TimeIntervalObject.from_dictionary(x) for x in dictionary.get('beats')]
        else:
            beats = APIHelper.SKIP
        sections = None
        if dictionary.get('sections') is not None:
            sections = [SectionObject.from_dictionary(x) for x in dictionary.get('sections')]
        else:
            sections = APIHelper.SKIP
        segments = None
        if dictionary.get('segments') is not None:
            segments = [SegmentObject.from_dictionary(x) for x in dictionary.get('segments')]
        else:
            segments = APIHelper.SKIP
        tatums = None
        if dictionary.get('tatums') is not None:
            tatums = [TimeIntervalObject.from_dictionary(x) for x in dictionary.get('tatums')]
        else:
            tatums = APIHelper.SKIP
        # Return an object of this model
        return cls(meta,
                   track,
                   bars,
                   beats,
                   sections,
                   segments,
                   tatums)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'meta={(self.meta if hasattr(self, "meta") else None)!r}, '
                f'track={(self.track if hasattr(self, "track") else None)!r}, '
                f'bars={(self.bars if hasattr(self, "bars") else None)!r}, '
                f'beats={(self.beats if hasattr(self, "beats") else None)!r}, '
                f'sections={(self.sections if hasattr(self, "sections") else None)!r}, '
                f'segments={(self.segments if hasattr(self, "segments") else None)!r}, '
                f'tatums={(self.tatums if hasattr(self, "tatums") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'meta={(self.meta if hasattr(self, "meta") else None)!s}, '
                f'track={(self.track if hasattr(self, "track") else None)!s}, '
                f'bars={(self.bars if hasattr(self, "bars") else None)!s}, '
                f'beats={(self.beats if hasattr(self, "beats") else None)!s}, '
                f'sections={(self.sections if hasattr(self, "sections") else None)!s}, '
                f'segments={(self.segments if hasattr(self, "segments") else None)!s}, '
                f'tatums={(self.tatums if hasattr(self, "tatums") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.artist_object import ArtistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_id_object import ExternalIdObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.linked_track_object import LinkedTrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.simplified_album_object import SimplifiedAlbumObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_restriction_object import TrackRestrictionObject


class TrackObject(object):

    """Implementation of the 'TrackObject' model.

    Attributes:
        album (SimplifiedAlbumObject): The album on which the track appears.
            The album object includes a link in `href` to full information
            about the album.
        artists (List[ArtistObject]): The artists who performed the track.
            Each artist object includes a link in `href` to more detailed
            information about the artist.
        available_markets (List[str]): A list of the countries in which the
            track can be played, identified by their [ISO 3166-1
            alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
        disc_number (int): The disc number (usually `1` unless the album
            consists of more than one disc).
        duration_ms (int): The track length in milliseconds.
        explicit (bool): Whether or not the track has explicit lyrics ( `true`
            = yes it does; `false` = no it does not OR unknown).
        external_ids (ExternalIdObject): Known external IDs for the track.
        external_urls (ExternalUrlObject): Known external URLs for this track.
        href (str): A link to the Web API endpoint providing full details of
            the track.
        id (str): The [Spotify
            ID](/documentation/web-api/concepts/spotify-uris-ids) for the
            track.
        is_playable (bool): Part of the response when [Track
            Relinking](/documentation/web-api/concepts/track-relinking) is
            applied. If `true`, the track is playable in the given market.
            Otherwise `false`.
        linked_from (LinkedTrackObject): Part of the response when [Track
            Relinking](/documentation/web-api/concepts/track-relinking) is
            applied, and the requested track has been replaced with different
            track. The track in the `linked_from` object contains information
            about the originally requested track.
        restrictions (TrackRestrictionObject): Included in the response when a
            content restriction is applied.
        name (str): The name of the track.
        popularity (int): The popularity of the track. The value will be
            between 0 and 100, with 100 being the most popular.<br/>The
            popularity of a track is a value between 0 and 100, with 100 being
            the most popular. The popularity is calculated by algorithm and is
            based, in the most part, on the total number of plays the track
            has had and how recent those plays are.<br/>Generally speaking,
            songs that are being played a lot now will have a higher
            popularity than songs that were played a lot in the past.
            Duplicate tracks (e.g. the same track from a single and an album)
            are rated independently. Artist and album popularity is derived
            mathematically from track popularity. _**Note**: the popularity
            value may lag actual popularity by a few days: the value is not
            updated in real time._
        preview_url (str): A link to a 30 second preview (MP3 format) of the
            track. Can be `null`
        track_number (int): The number of the track. If an album has several
            discs, the track number is the number on the specified disc.
        mtype (Type2Enum): The object type: "track".
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for the
            track.
        is_local (bool): Whether or not the track is from a local file.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "album": 'album',
        "artists": 'artists',
        "available_markets": 'available_markets',
        "disc_number": 'disc_number',
        "duration_ms": 'duration_ms',
        "explicit": 'explicit',
        "external_ids": 'external_ids',
        "external_urls": 'external_urls',
        "href": 'href',
        "id": 'id',
        "is_playable": 'is_playable',
        "linked_from": 'linked_from',
        "restrictions": 'restrictions',
        "name": 'name',
        "popularity": 'popularity',
        "preview_url": 'preview_url',
        "track_number": 'track_number',
        "mtype": 'type',
        "uri": 'uri',
        "is_local": 'is_local'
    }

    _optionals = [
        'album',
        'artists',
        'available_markets',
        'disc_number',
        'duration_ms',
        'explicit',
        'external_ids',
        'external_urls',
        'href',
        'id',
        'is_playable',
        'linked_from',
        'restrictions',
        'name',
        'popularity',
        'preview_url',
        'track_number',
        'mtype',
        'uri',
        'is_local',
    ]

    _nullables = [
        'preview_url',
    ]

    def __init__(self,
                 album=APIHelper.SKIP,
                 artists=APIHelper.SKIP,
                 available_markets=APIHelper.SKIP,
                 disc_number=APIHelper.SKIP,
                 duration_ms=APIHelper.SKIP,
                 explicit=APIHelper.SKIP,
                 external_ids=APIHelper.SKIP,
                 external_urls=APIHelper.SKIP,
                 href=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 is_playable=APIHelper.SKIP,
                 linked_from=APIHelper.SKIP,
                 restrictions=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 popularity=APIHelper.SKIP,
                 preview_url=APIHelper.SKIP,
                 track_number=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 uri=APIHelper.SKIP,
                 is_local=APIHelper.SKIP):
        """Constructor for the TrackObject class"""

        # Initialize members of the class
        if album is not APIHelper.SKIP:
            self.album = album 
        if artists is not APIHelper.SKIP:
            self.artists = artists 
        if available_markets is not APIHelper.SKIP:
            self.available_markets = available_markets 
        if disc_number is not APIHelper.SKIP:
            self.disc_number = disc_number 
        if duration_ms is not APIHelper.SKIP:
            self.duration_ms = duration_ms 
        if explicit is not APIHelper.SKIP:
            self.explicit = explicit 
        if external_ids is not APIHelper.SKIP:
            self.external_ids = external_ids 
        if external_urls is not APIHelper.SKIP:
            self.external_urls = external_urls 
        if href is not APIHelper.SKIP:
            self.href = href 
        if id is not APIHelper.SKIP:
            self.id = id 
        if is_playable is not APIHelper.SKIP:
            self.is_playable = is_playable 
        if linked_from is not APIHelper.SKIP:
            self.linked_from = linked_from 
        if restrictions is not APIHelper.SKIP:
            self.restrictions = restrictions 
        if name is not APIHelper.SKIP:
            self.name = name 
        if popularity is not APIHelper.SKIP:
            self.popularity = popularity 
        if preview_url is not APIHelper.SKIP:
            self.preview_url = preview_url 
        if track_number is not APIHelper.SKIP:
            self.track_number = track_number 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if uri is not APIHelper.SKIP:
            self.uri = uri 
        if is_local is not APIHelper.SKIP:
            self.is_local = is_local 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        album = SimplifiedAlbumObject.from_dictionary(dictionary.get('album')) if 'album' in dictionary.keys() else APIHelper.SKIP
        artists = None
        if dictionary.get('artists') is not None:
            artists = [ArtistObject.from_dictionary(x) for x in dictionary.get('artists')]
        else:
            artists = APIHelper.SKIP
        available_markets = dictionary.get("available_markets") if dictionary.get("available_markets") else APIHelper.SKIP
        disc_number = dictionary.get("disc_number") if dictionary.get("disc_number") else APIHelper.SKIP
        duration_ms = dictionary.get("duration_ms") if dictionary.get("duration_ms") else APIHelper.SKIP
        explicit = dictionary.get("explicit") if "explicit" in dictionary.keys() else APIHelper.SKIP
        external_ids = ExternalIdObject.from_dictionary(dictionary.get('external_ids')) if 'external_ids' in dictionary.keys() else APIHelper.SKIP
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if 'external_urls' in dictionary.keys() else APIHelper.SKIP
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        is_playable = dictionary.get("is_playable") if "is_playable" in dictionary.keys() else APIHelper.SKIP
        linked_from = LinkedTrackObject.from_dictionary(dictionary.get('linked_from')) if 'linked_from' in dictionary.keys() else APIHelper.SKIP
        restrictions = TrackRestrictionObject.from_dictionary(dictionary.get('restrictions')) if 'restrictions' in dictionary.keys() else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        popularity = dictionary.get("popularity") if dictionary.get("popularity") else APIHelper.SKIP
        preview_url = dictionary.get("preview_url") if "preview_url" in dictionary.keys() else APIHelper.SKIP
        track_number = dictionary.get("track_number") if dictionary.get("track_number") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        uri = dictionary.get("uri") if dictionary.get("uri") else APIHelper.SKIP
        is_local = dictionary.get("is_local") if "is_local" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(album,
                   artists,
                   available_markets,
                   disc_number,
                   duration_ms,
                   explicit,
                   external_ids,
                   external_urls,
                   href,
                   id,
                   is_playable,
                   linked_from,
                   restrictions,
                   name,
                   popularity,
                   preview_url,
                   track_number,
                   mtype,
                   uri,
                   is_local)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return True

        if not isinstance(dictionary, dict):
            return False

        return True

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'album={(self.album if hasattr(self, "album") else None)!r}, '
                f'artists={(self.artists if hasattr(self, "artists") else None)!r}, '
                f'available_markets={(self.available_markets if hasattr(self, "available_markets") else None)!r}, '
                f'disc_number={(self.disc_number if hasattr(self, "disc_number") else None)!r}, '
                f'duration_ms={(self.duration_ms if hasattr(self, "duration_ms") else None)!r}, '
                f'explicit={(self.explicit if hasattr(self, "explicit") else None)!r}, '
                f'external_ids={(self.external_ids if hasattr(self, "external_ids") else None)!r}, '
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!r}, '
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'is_playable={(self.is_playable if hasattr(self, "is_playable") else None)!r}, '
                f'linked_from={(self.linked_from if hasattr(self, "linked_from") else None)!r}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'popularity={(self.popularity if hasattr(self, "popularity") else None)!r}, '
                f'preview_url={(self.preview_url if hasattr(self, "preview_url") else None)!r}, '
                f'track_number={(self.track_number if hasattr(self, "track_number") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!r}, '
                f'is_local={(self.is_local if hasattr(self, "is_local") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'album={(self.album if hasattr(self, "album") else None)!s}, '
                f'artists={(self.artists if hasattr(self, "artists") else None)!s}, '
                f'available_markets={(self.available_markets if hasattr(self, "available_markets") else None)!s}, '
                f'disc_number={(self.disc_number if hasattr(self, "disc_number") else None)!s}, '
                f'duration_ms={(self.duration_ms if hasattr(self, "duration_ms") else None)!s}, '
                f'explicit={(self.explicit if hasattr(self, "explicit") else None)!s}, '
                f'external_ids={(self.external_ids if hasattr(self, "external_ids") else None)!s}, '
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!s}, '
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'is_playable={(self.is_playable if hasattr(self, "is_playable") else None)!s}, '
                f'linked_from={(self.linked_from if hasattr(self, "linked_from") else None)!s}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'popularity={(self.popularity if hasattr(self, "popularity") else None)!s}, '
                f'preview_url={(self.preview_url if hasattr(self, "preview_url") else None)!s}, '
                f'track_number={(self.track_number if hasattr(self, "track_number") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!s}, '
                f'is_local={(self.is_local if hasattr(self, "is_local") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ItemType1Enum(object):

    """Implementation of the 'Item Type1' enum.

    The ID type: currently only `artist` is supported.

    Attributes:
        ARTIST: The enum member of type str.

    """
    _all_values = ['artist']
    ARTIST = 'artist'

    @classmethod
    def validate(cls, value):
        """Validates value contains in enum

        Args:
            value: the value to be validated

        Returns:
            boolean : if value is valid enum values.

        """
        return value in cls._all_values
   


# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class TimeIntervalObject(object):

    """Implementation of the 'TimeIntervalObject' model.

    Attributes:
        start (float): The starting point (in seconds) of the time interval.
        duration (float): The duration (in seconds) of the time interval.
        confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the interval.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "start": 'start',
        "duration": 'duration',
        "confidence": 'confidence'
    }

    _optionals = [
        'start',
        'duration',
        'confidence',
    ]

    def __init__(self,
                 start=APIHelper.SKIP,
                 duration=APIHelper.SKIP,
                 confidence=APIHelper.SKIP):
        """Constructor for the TimeIntervalObject class"""

        # Initialize members of the class
        if start is not APIHelper.SKIP:
            self.start = start 
        if duration is not APIHelper.SKIP:
            self.duration = duration 
        if confidence is not APIHelper.SKIP:
            self.confidence = confidence 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        start = dictionary.get("start") if dictionary.get("start") else APIHelper.SKIP
        duration = dictionary.get("duration") if dictionary.get("duration") else APIHelper.SKIP
        confidence = dictionary.get("confidence") if dictionary.get("confidence") else APIHelper.SKIP
        # Return an object of this model
        return cls(start,
                   duration,
                   confidence)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'start={(self.start if hasattr(self, "start") else None)!r}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!r}, '
                f'confidence={(self.confidence if hasattr(self, "confidence") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'start={(self.start if hasattr(self, "start") else None)!s}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!s}, '
                f'confidence={(self.confidence if hasattr(self, "confidence") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class DeviceObject(object):

    """Implementation of the 'DeviceObject' model.

    Attributes:
        id (str): The device ID. This ID is unique and persistent to some
            extent. However, this is not guaranteed and any cached `device_id`
            should periodically be cleared out and refetched as necessary.
        is_active (bool): If this device is the currently active device.
        is_private_session (bool): If this device is currently in a private
            session.
        is_restricted (bool): Whether controlling this device is restricted.
            At present if this is "true" then no Web API commands will be
            accepted by this device.
        name (str): A human-readable name for the device. Some devices have a
            name that the user can configure (e.g. \"Loudest speaker\") and
            some devices have a generic name associated with the manufacturer
            or device model.
        mtype (str): Device type, such as "computer", "smartphone" or
            "speaker".
        volume_percent (int): The current volume in percent.
        supports_volume (bool): If this device can be used to set the volume.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "is_active": 'is_active',
        "is_private_session": 'is_private_session',
        "is_restricted": 'is_restricted',
        "name": 'name',
        "mtype": 'type',
        "volume_percent": 'volume_percent',
        "supports_volume": 'supports_volume'
    }

    _optionals = [
        'id',
        'is_active',
        'is_private_session',
        'is_restricted',
        'name',
        'mtype',
        'volume_percent',
        'supports_volume',
    ]

    _nullables = [
        'id',
        'volume_percent',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 is_active=APIHelper.SKIP,
                 is_private_session=APIHelper.SKIP,
                 is_restricted=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 volume_percent=APIHelper.SKIP,
                 supports_volume=APIHelper.SKIP):
        """Constructor for the DeviceObject class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if is_active is not APIHelper.SKIP:
            self.is_active = is_active 
        if is_private_session is not APIHelper.SKIP:
            self.is_private_session = is_private_session 
        if is_restricted is not APIHelper.SKIP:
            self.is_restricted = is_restricted 
        if name is not APIHelper.SKIP:
            self.name = name 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if volume_percent is not APIHelper.SKIP:
            self.volume_percent = volume_percent 
        if supports_volume is not APIHelper.SKIP:
            self.supports_volume = supports_volume 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if "id" in dictionary.keys() else APIHelper.SKIP
        is_active = dictionary.get("is_active") if "is_active" in dictionary.keys() else APIHelper.SKIP
        is_private_session = dictionary.get("is_private_session") if "is_private_session" in dictionary.keys() else APIHelper.SKIP
        is_restricted = dictionary.get("is_restricted") if "is_restricted" in dictionary.keys() else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        volume_percent = dictionary.get("volume_percent") if "volume_percent" in dictionary.keys() else APIHelper.SKIP
        supports_volume = dictionary.get("supports_volume") if "supports_volume" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   is_active,
                   is_private_session,
                   is_restricted,
                   name,
                   mtype,
                   volume_percent,
                   supports_volume)

    @classmethod
    def validate(cls, dictionary):
        """Validates dictionary against class required properties

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            boolean : if dictionary is valid contains required properties.

        """

        if isinstance(dictionary, cls):
            return True

        if not isinstance(dictionary, dict):
            return False

        return True

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'is_active={(self.is_active if hasattr(self, "is_active") else None)!r}, '
                f'is_private_session={(self.is_private_session if hasattr(self, "is_private_session") else None)!r}, '
                f'is_restricted={(self.is_restricted if hasattr(self, "is_restricted") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'volume_percent={(self.volume_percent if hasattr(self, "volume_percent") else None)!r}, '
                f'supports_volume={(self.supports_volume if hasattr(self, "supports_volume") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'is_active={(self.is_active if hasattr(self, "is_active") else None)!s}, '
                f'is_private_session={(self.is_private_session if hasattr(self, "is_private_session") else None)!s}, '
                f'is_restricted={(self.is_restricted if hasattr(self, "is_restricted") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'volume_percent={(self.volume_percent if hasattr(self, "volume_percent") else None)!s}, '
                f'supports_volume={(self.supports_volume if hasattr(self, "supports_volume") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.http.request.http_request import HttpRequest


class HttpRequest(HttpRequest):

    """Information about an HTTP Request including its method, headers,
        parameters, URL, and Basic Auth details

    Attributes:
        http_method (HttpMethodEnum): The HTTP Method that this request should
            perform when called.
        headers (dict): A dictionary of headers (key : value) that should be
            sent along with the request.
        query_url (string): The URL that the request should be sent to.
        parameters (dict): A dictionary of parameters that are to be sent along
            with the request in the form body of the request

    """

    def __init__(self,
                 http_method,
                 query_url,
                 headers=None,
                 query_parameters=None,
                 parameters=None,
                 files=None):
        """Constructor for the HttpRequest class

        Args:
            http_method (HttpMethodEnum): The HTTP Method.
            query_url (string): The URL to send the request to.
            headers (dict, optional): The headers for the HTTP Request.
            query_parameters (dict, optional): Query parameters to add in the
                URL.
            parameters (dict, optional): Form or body parameters to be included
                in the body.
            files (dict, optional): Files to be sent with the request.

        """
        super().__init__(http_method,
                         query_url,
                         headers,
                         query_parameters,
                         parameters,
                         files)
